﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace AitoffProjection
{
    class HealPix
    {
        const int ns_max = 8192;

        const double HALFPI = 1.570796326794896619231321691639751442099;
        const double TWOPI = 6.283185307179586476925286766559005768394;
        const double TWOTHIRD = 0.6666666666666666666666666666666666666666;

        public static int ang2pix_ring(int nside, double phi, double theta)
        {
          int  nl4, jp, jm;
          double z, za, tt, tp, tmp, temp1, temp2;
          int ir, ip, kshift, ipix;

          if (nside<1 || nside>ns_max) throw new IndexOutOfRangeException("ang2pix_ring: nside out of range");
          if (theta<-HALFPI || theta>HALFPI)  throw new IndexOutOfRangeException("ANG2PIX_RING: theta : "+theta.ToString()+" is out of range [-Pi/2, +Pi/2]");

          z = Math.Cos(theta+HALFPI);
          za = Math.Abs(z);
 
          if( phi >= TWOPI)  phi = phi - TWOPI;
          if (phi < 0.0)     phi = phi + TWOPI;
          tt = phi / HALFPI;//  ! in [0,4)

          if (za <= TWOTHIRD) // Equatorial region ------------------
          {
              temp1 = nside * (0.5 + tt);
              temp2 = nside * 0.75 * z;
              jp = (int)(temp1 - temp2); // index of  ascending edge line
              jm = (int)(temp1 + temp2); // index of descending edge line

              ir = nside + 1 + jp - jm; // in {1,2n+1} (ring number counted from z=2/3)
              kshift = 1 - ir % 2;        // kshift=1 if ir even, 0 otherwise

              nl4 = 4 * nside;
              ip = (int)((jp + jm - nside + kshift + 1) / 2); // in {0,4n-1}
              if (ip >= nl4) ip = ip - nl4;

              ipix = 2 * nside * (nside - 1) + nl4 * (ir - 1) + ip;
          }
          else
          {
              tp = tt - Math.Floor(tt);    //  !MODULO(tt,1.0_8)
              tmp = nside * Math.Sqrt(3.0 * (1.0 - za));

              jp = (int)(tp * tmp); // ! increasing edge line index
              jm = (int)((1.0 - tp) * tmp); // ! decreasing edge line index

              ir = jp + jm + 1; //        ! ring number counted from the closest pole
              ip = (int)(tt * ir); //     ! in {0,4*ir-1}
              if (ip >= 4 * ir) ip = ip - 4 * ir;

              if (z > 0.0)
                  ipix = 2 * ir * (ir - 1) + ip;
              else
                  ipix = 12 * nside * nside - 2 * ir * (ir + 1) + ip;
          }
          return ipix;
        }
    }
}
